lasso<-function(C,X,Z){
 
  # C: M x N (target site information matrix)
  # X: M x T (mRNA express profile matrix )
  # Z: N x T (miRNA express profile matrix ) 
  
  oz <- t(Z)  # T x N
  
  # run LASSO to construct W
  W <- lapply(1:nrow(X), function(i) {				
    
    pred <- matrix(rep(0, nrow(Z)), nrow=1,
                   dimnames=list(rownames(X)[i], rownames(Z)))
    
    ci <- t(matrix(rep(C[i,,drop=FALSE], nrow(oz)), ncol=nrow(oz)))
    
    ci <- (ci > 0) + 0 # convert to binary matrix
    
    inp <- oz * ci
    
    # use only miRNA with at least one non-zero entry across T samples
    inp <- inp[, apply(abs(inp), 2, max)>0, drop=FALSE]
    
    if(ncol(inp) >= 2) {
      
      # NOTE: negative coef means potential miRNA target 
      y <- coef(cv.glmnet(inp, X[i,], nfolds=10), s="lambda.min")[-1]
      
      pred[, match(colnames(inp), colnames(pred))] <- y
    }
    pred[pred>0] <- 0
    
    pred <- abs(pred)
    
    pred[pred>1] <- 1	
    
    pred
  })
  W <- do.call("rbind", W)

  W
}

procData<-function(W,filename){ 
  #convert to number for easy process
  dimnames(W) <- list(1:nrow(W), 1:ncol(W))

  #get every miRNA with edges
  n.list <- lapply(split(W, as.numeric(rownames(W))), 	
                   function(x) as.numeric(colnames(W)[which(x>0)]))	
  fdata<-c()
  sdata<-c()
  #traversal to get edges
  for(i in 1:length(n.list)) {
    len<-length(n.list[[i]])
    firstpart<-rep(i,len)
    secondpart<-n.list[[i]]
    fdata<-c(fdata,firstpart)
    sdata<-c(sdata,secondpart)
  }
  mdata<-matrix(c(fdata,sdata),ncol = 2)
  #save
  write.table(mdata,filename,quote = FALSE, sep =" ",
              eol = "\n",row.names = FALSE,col.names = FALSE)
}


readFile<-function(filename){
    #read data and sort by miRNA
    con <- file(filename, "r")
    #the first row are mRNAs
    line=readLines(con,n=1)
    #keep miRNAs and mRNAs in biclique
    V_miR<-list()
    V_mR<-list()
    while(length(line) != 0 ) {
      mrna<-strsplit(line," ")
      #the second row are miRNAs
      line=readLines(con,n=1)
      mirna<-strsplit(line," ")
  
      V_miR<-c(V_miR,mirna)
      V_mR<-c(V_mR,mrna)
      
      #step over the third line which is a blank line
      line=readLines(con,n=1)
      #read next row
      line=readLines(con,n=1)
    }
    close(con)
    mdata<-c(list(V_mR),list(V_miR))
    return(mdata)
}


buildPPIMatrix<-function(ppi,mRNAs,C){
  #according mRNA's names,find position in mRNAs
  #put the interaction into newppi
  colnames(C)<-c(1:ncol(C))
  rownames(C)<-c(1:nrow(C))
  newppi<-list()
  for(i in 1:length(ppi[["GENE1"]])){
    gene1<-as.vector(ppi[["GENE1"]][[i]])
    gene2<-as.vector(ppi[["GENE2"]][[i]])
    gg<-c(gene1,gene2)
    if(gene1 %in% mRNAs)
      newppi<-c(newppi,list(gg))
  }
  #newppi2 is indexes of gene in newppi
  newppi2<-list()
  for(i in 1:length(newppi)){
    gene1<-newppi[[i]][1]
    gene2<-newppi[[i]][2]
    if(gene2 %in% mRNAs){
      gg<-c(which(mRNAs==gene1),which(mRNAs==gene2))
      newppi2<-c(newppi2,list(gg))
    }
  }
  
  
  #PPI adject matrix
  ppimatrix<-c()
  for(i in 1:length(mRNAs)){
    genelist<-c()
    for(j in 1:length(newppi2)){
      if(i==newppi2[[j]][1]){
        genelist<-c(genelist,newppi2[[j]][2])
      }
      if(i==newppi2[[j]][2]){
        genelist<-c(genelist,newppi2[[j]][1])
      }
    }
    ppimatrix<-c(ppimatrix,list(genelist))
  }
  #Remove duplicate edges
  for(i in 1:length(ppimatrix)){
    ppimatrix[[i]]<-as.vector(levels(factor(ppimatrix[[i]])))
  }
  #Remove self-loop edges
  for(i in 1:length(ppimatrix)){
    if(i %in% ppimatrix[[i]])
      ppimatrix[[i]]<-setdiff(ppimatrix[[i]],c(as.character(i)))
    
  }
  
  ppimatrix
}



writeCMFMToFile<-function(resultMatrix,miRNAs,mRNAs,filename){
  miRNACol<-c()
  mRNACol<-c()
  for(i in 1:length(resultMatrix[,1])){
    miRNA<-miRNAs[as.integer(resultMatrix[,"miRNA"][[i]])]
    strMiRNA<-miRNA[1]
    for(j in 2:length(miRNA)){
      strMiRNA<-paste(strMiRNA,miRNA[j],sep=",")
    }
    miRNACol<-c(miRNACol,strMiRNA)
    
    mRNA<-mRNAs[as.integer(resultMatrix[,"mRNA"][[i]])]
    strMRNA<-mRNA[1]
    for(j in 2:length(mRNA)){
      strMRNA<-paste(strMRNA,mRNA[j],sep=",")
    }
    mRNACol<-c(mRNACol,strMRNA)
  }
  
  V<-cbind(rep(1:length(resultMatrix[,1])),miRNACol,mRNACol)
  colnames(V)<-c("module","miRNA","mRNA")

  #writeFile(fmdata,filename)
  write.csv(V,filename,row.names = FALSE,col.names = TRUE)

}

